(function() {
  var Events, Freakset;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  Events = require('events');
  Freakset = (function() {
    __extends(Freakset, Events.EventEmitter);
    Freakset.Actor = {};
    Freakset.Actor.Group = require('./freakset/actor/group').Actor;
    Freakset.Actor.Guard = require('./freakset/actor/guard').Actor;
    Freakset.Actor.Rescue = require('./freakset/actor/rescue').Actor;
    Freakset.Actor.Step = require('./freakset/actor/step').Actor;
    Freakset.build = function(opts) {
      return new Freakset(opts || (opts = {}));
    };
    function Freakset(opts) {
      opts || (opts = {});
      this.klass = Freakset;
      this.stack = opts.stack || (opts.stack = []);
      this.store = opts.store || (opts.store = {});
      this.outerFreakset = opts.outerFreakset || (opts.outerFreakset = void 0);
      this.resolvedCallback = opts.resolvedCallback || (opts.resolvedCallback = void 0);
      this.runInParallel = opts.runInParallel || (opts.runInParallel = void 0);
      this.running = false;
      this.modificationIndex = 0;
      this.innerFreaksets = [];
    }
    Freakset.prototype.step = function(scope, opts, fn) {
      return Freakset.Actor.Step.build(this, scope, opts, fn);
    };
    Freakset.prototype.group = function(scope, opts, fn) {
      return Freakset.Actor.Group.build(this, scope, opts, fn);
    };
    Freakset.prototype.rescue = function(scope, errFn, opts, fn) {
      return Freakset.Actor.Rescue.build(this, scope, errFn, opts, fn);
    };
    Freakset.prototype.guard = function(scope, opts, fn) {
      return Freakset.Actor.Guard.build(this, scope, opts, fn);
    };
    Freakset.prototype.isRunning = function() {
      var innerFreakset, _i, _len, _ref;
      if (this.running === true) {
        return true;
      }
      if (this.innerFreaksets.length !== 0) {
        _ref = this.innerFreaksets;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          innerFreakset = _ref[_i];
          if (innerFreakset.isRunning()) {
            return true;
          }
        }
      }
      return false;
    };
    Freakset.prototype.root = function() {
      if (this.outerFreakset != null) {
        return this.outerFreakset.root();
      } else {
        return this;
      }
    };
    Freakset.prototype.next = function() {
      var actor;
      if (this.root().canceled != null) {
        return false;
      }
      this.running = true;
      actor = this.stack.shift();
      if (actor == null) {
        if (this.runInParallel != null) {
          if ((this.parallelStepCount != null) && this.parallelStepCount === 0) {
            return this.finish();
          } else {
            return;
          }
        } else {
          return this.finish();
        }
      }
      this.emit("beforeRunningActor", this, actor);
      this.emit("before:" + actor.scope);
      switch (actor.type) {
        case 'group':
          this.runActor(Freakset.Actor.Group.run, actor);
          break;
        case 'guard':
          this.runActor(Freakset.Actor.Guard.run, actor);
          break;
        case 'rescue':
          this.runActor(Freakset.Actor.Rescue.run, actor);
          break;
        case 'step':
          this.runActor(Freakset.Actor.Step.run, actor);
      }
      this.emit("afterRunningActor", this, actor);
      return this.emit("after:" + actor.scope);
    };
    Freakset.prototype.commit = function() {
      if ((this.runInParallel != null) && this.parallelStepCount) {
        this.parallelStepCount -= 1;
      }
      return this.next();
    };
    Freakset.prototype.compile = function(store, resolvedCallback) {
      var set;
      if (resolvedCallback != null) {
        this.resolvedCallback = resolvedCallback;
      }
      store || (store = {});
      set = this.buildAndRunFreakset({
        stack: this.stack,
        store: store,
        resolvedCallback: resolvedCallback
      });
      return set;
    };
    Freakset.prototype.cancel = function() {
      this.root().canceled = true;
      this.root().running = false;
      return this.root().resolveRoot();
    };
    Freakset.prototype.finish = function() {
      if (this.root().canceled == null) {
        this.running = false;
        this.resolveRoot();
        if (this.outerFreakset) {
          this.outerFreakset.commit();
        }
        return true;
      }
    };
    Freakset.prototype.resolveRoot = function() {
      var root;
      root = this.root();
      if (!root.isRunning()) {
        this.emit('finish', root);
        if (root.resolvedCallback != null) {
          return root.resolvedCallback.call(root);
        }
      }
    };
    Freakset.prototype.hasActor = function(scope) {
      return (_.detect(this.stack, function(actor) {
        return actor.scope === scope;
      })) != null;
    };
    Freakset.prototype.buildAndRunFreakset = function(opts) {
      var freakset;
      if (opts.stack != null) {
        opts.stack = opts.stack.slice();
      }
      freakset = Freakset.build(opts);
      this.copyEventListeners(freakset);
      if (freakset === freakset.root()) {
        this.emit('compile', freakset);
      }
      freakset.next();
      return freakset;
    };
    Freakset.prototype.copyEventListeners = function(freakset) {
      var event, listener, _results;
      if (this._events != null) {
        _results = [];
        for (event in this._events) {
          _results.push((function() {
            var _i, _len, _ref, _results;
            _ref = this.listeners(event);
            _results = [];
            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
              listener = _ref[_i];
              _results.push(freakset.on(event, listener));
            }
            return _results;
          }).call(this));
        }
        return _results;
      }
    };
    Freakset.prototype.runActor = function(runner, actor) {
      var chainableEventFns;
      chainableEventFns = [];
      this.modificationIndex = 0;
      if (this._events != null) {
        chainableEventFns = this.findChainableEventFns('aroundRunningActor');
        chainableEventFns = chainableEventFns.concat(this.findChainableEventFns("around:" + actor.scope));
      }
      if (chainableEventFns.length > 0) {
        return this.runEventFnChain(chainableEventFns, runner, actor);
      } else {
        return runner(this, actor);
      }
    };
    Freakset.prototype.findChainableEventFns = function(eventName) {
      if ((this._events[eventName] != null) && (typeof this._events[eventName] === 'function' || this._events[eventName].length > 0)) {
        if (typeof this._events[eventName] === 'function') {
          return [this._events[eventName]];
        } else {
          return _.clone(this._events[eventName]);
        }
      } else {
        return [];
      }
    };
    Freakset.prototype.runEventFnChain = function(eventFns, runner, actor) {
      var eventFn;
      eventFn = eventFns.shift();
      return eventFn(this, actor, __bind(function() {
        if (eventFns.length > 0) {
          return this.runEventFnChain(eventFns, runner, actor);
        } else {
          return runner(this, actor);
        }
      }, this));
    };
    Freakset.prototype.modifyStack = function(set, actor, opts) {
      var index, _ref, _ref2, _ref3;
      opts || (opts = {});
      if (!opts.insertAt && !opts.insertAtTop && !opts.insertBefore && !opts.insertAfter && !opts.replace && this.isRunning()) {
        opts.insertAt = this.modificationIndex;
        this.modificationIndex += 1;
      }
      if ((opts.insertAt != null) && typeof opts.insertAt === 'number') {
        return set.stack.splice(opts.insertAt, 0, actor);
      } else if ((opts.insertAtTop != null) && opts.insertAtTop === true) {
        return set.stack.unshift(actor);
      } else if ((opts.insertBefore != null) && typeof opts.insertBefore === "string") {
        _ref = this.findActorByScope(set, opts.insertBefore), index = _ref[0], set = _ref[1];
        if (index !== -1) {
          return set.stack.splice(index, 0, actor);
        }
      } else if ((opts.insertAfter != null) && typeof opts.insertAfter === "string") {
        _ref2 = this.findActorByScope(set, opts.insertAfter), index = _ref2[0], set = _ref2[1];
        if (index !== -1) {
          return set.stack.splice(index + 1, 0, actor);
        }
      } else if ((opts.replace != null) && typeof opts.replace === "string") {
        _ref3 = this.findActorByScope(set, opts.replace), index = _ref3[0], set = _ref3[1];
        if (index !== -1) {
          return set.stack.splice(index, 1, actor);
        }
      } else {
        return set.stack.push(actor);
      }
    };
    Freakset.prototype.findActorByScope = function(set, scope) {
      var actor, index, innerSet, _i, _len, _ref, _ref2;
      if (set == null) {
        set = this;
      }
      actor = _.detect(set.stack, function(actor) {
        return actor.scope === scope;
      });
      if (actor != null) {
        return [set.stack.indexOf(actor), set];
      } else {
        _ref = set.innerFreaksets;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          innerSet = _ref[_i];
          _ref2 = this.findActorByScope(innerSet, scope), index = _ref2[0], set = _ref2[1];
          if (index !== -1) {
            return [index, set];
          }
        }
        return [-1, null];
      }
    };
    return Freakset;
  })();
  exports.Freakset = Freakset;
}).call(this);
